@@ -65,15 +65,18 @@ class ReportExpectMock:
     def __getattr__(self, name):
         if name[0] == "_":
             raise AttributeError(name)
+        elif name == 'verbosity':
+            # FIXME: special case for property on Reporter class, may it be generalized?
+            return 0
 
         def generic_report(*args, **kwargs):
             self._calls.append((name,) + args)
-            print ("%s" % (self._calls[-1], ))
+            print("%s" % (self._calls[-1], ))
         return generic_report
 
     def action(self, venv, msg, *args):
         self._calls.append(("action", venv, msg))
-        print ("%s" % (self._calls[-1], ))
+        print("%s" % (self._calls[-1], ))
         return Action(self.session, venv, msg, args)
 
     def getnext(self, cat):
@@ -195,6 +198,9 @@ class Cmd:
         return py.std.subprocess.Popen(argv, stdout=stdout, stderr=stderr, **kw)
 
     def run(self, *argv):
+        if argv[0] == "tox" and sys.version_info[:2] < (2, 7):
+            pytest.skip("can not run tests involving calling tox on python2.6. "
+                        "(and python2.6 is about to be deprecated anyway)")
         argv = [str(x) for x in argv]
         assert py.path.local.sysfind(str(argv[0])), argv[0]
         p1 = self.tmpdir.join("stdout")
@@ -284,7 +290,7 @@ class LineMatcher:
 @pytest.fixture
 def initproj(request, tmpdir):
     """ create a factory function for creating example projects. """
-    def initproj(nameversion, filedefs=None):
+    def initproj(nameversion, filedefs=None, src_root="."):
         if filedefs is None:
             filedefs = {}
         if _istext(nameversion) or _isbytes(nameversion):
@@ -298,25 +304,27 @@ def initproj(request, tmpdir):
         create_files(base, filedefs)
         if 'setup.py' not in filedefs:
             create_files(base, {'setup.py': '''
-                from setuptools import setup
+                from setuptools import setup, find_packages
                 setup(
                     name='%(name)s',
                     description='%(name)s project',
                     version='%(version)s',
                     license='MIT',
                     platforms=['unix', 'win32'],
-                    packages=['%(name)s', ],
+                    packages=find_packages('%(src_root)s'),
+                    package_dir={'':'%(src_root)s'},
                 )
             ''' % locals()})
         if name not in filedefs:
-            create_files(base, {
+            src_dir = base.ensure(src_root, dir=1)
+            create_files(src_dir, {
                 name: {'__init__.py': '__version__ = %r' % version}
             })
         manifestlines = []
         for p in base.visit(lambda x: x.check(file=1)):
             manifestlines.append("include %s" % p.relto(base))
         create_files(base, {"MANIFEST.in": "\n".join(manifestlines)})
-        print ("created project in %s" % (base,))
+        print("created project in %s" % (base,))
         base.chdir()
     return initproj
 
