@@ -9,6 +9,7 @@ import string
 import pkg_resources
 import itertools
 import pluggy
+from subprocess import list2cmdline
 
 import tox.interpreters
 from tox import hookspecs
@@ -21,8 +22,8 @@ import tox
 iswin32 = sys.platform == "win32"
 
 default_factors = {'jython': 'jython', 'pypy': 'pypy', 'pypy3': 'pypy3',
-                   'py': sys.executable}
-for version in '26,27,32,33,34,35,36'.split(','):
+                   'py': sys.executable, 'py2': 'python2', 'py3': 'python3'}
+for version in '26,27,32,33,34,35,36,37'.split(','):
     default_factors['py' + version] = 'python%s.%s' % tuple(version)
 
 hookimpl = pluggy.HookimplMarker("tox")
@@ -218,8 +219,11 @@ def parseconfig(args=None, plugins=()):
 
     # parse ini file
     basename = config.option.configfile
-    if os.path.isabs(basename):
+    if os.path.isfile(basename):
         inipath = py.path.local(basename)
+    elif os.path.isdir(basename):
+        # Assume 'tox.ini' filename if directory was passed
+        inipath = py.path.local(os.path.join(basename, 'tox.ini'))
     else:
         for path in py.path.local().parts(reverse=True):
             inipath = path.join(basename)
@@ -228,7 +232,11 @@ def parseconfig(args=None, plugins=()):
         else:
             inipath = py.path.local().join('setup.cfg')
             if not inipath.check():
-                feedback("toxini file %r not found" % (basename), sysexit=True)
+                helpoptions = option.help or option.helpini
+                feedback("toxini file %r not found" % (basename),
+                         sysexit=not helpoptions)
+                if helpoptions:
+                    return config
 
     try:
         parseini(config, inipath)
@@ -256,14 +264,6 @@ class VersionAction(argparse.Action):
         raise SystemExit(0)
 
 
-class CountAction(argparse.Action):
-    def __call__(self, parser, namespace, values, option_string=None):
-        if hasattr(namespace, self.dest):
-            setattr(namespace, self.dest, int(getattr(namespace, self.dest)) + 1)
-        else:
-            setattr(namespace, self.dest, 0)
-
-
 class SetenvDict:
     def __init__(self, dict, reader):
         self.reader = reader
@@ -315,16 +315,21 @@ def tox_addoption(parser):
                         help="show help about options")
     parser.add_argument("--help-ini", "--hi", action="store_true", dest="helpini",
                         help="show help about ini-names")
-    parser.add_argument("-v", nargs=0, action=CountAction, default=0,
-                        dest="verbosity",
-                        help="increase verbosity of reporting output.")
+    parser.add_argument("-v", action='count', dest="verbosity", default=0,
+                        help="increase verbosity of reporting output. -vv mode turns off "
+                        "output redirection for package installation")
     parser.add_argument("--showconfig", action="store_true",
                         help="show configuration information for all environments. ")
     parser.add_argument("-l", "--listenvs", action="store_true",
-                        dest="listenvs", help="show list of test environments")
+                        dest="listenvs", help="show list of test environments "
+                                              "(with description if verbose)")
+    parser.add_argument("-a", "--listenvs-all", action="store_true",
+                        dest="listenvs_all",
+                        help="show list of all defined environments"
+                             "(with description if verbose)")
     parser.add_argument("-c", action="store", default="tox.ini",
                         dest="configfile",
-                        help="use the specified config file name.")
+                        help="config file name or directory with 'tox.ini' file.")
     parser.add_argument("-e", action="append", dest="env",
                         metavar="envlist",
                         help="work against specified environments (ALL selects all).")
@@ -369,6 +374,8 @@ def tox_addoption(parser):
                              "'pytest<2.7' or 'django>=1.6'.")
     parser.add_argument("--sitepackages", action="store_true",
                         help="override sitepackages setting to True in all envs")
+    parser.add_argument("--alwayscopy", action="store_true",
+                        help="override alwayscopy setting to True in all envs")
     parser.add_argument("--skip-missing-interpreters", action="store_true",
                         help="don't fail tests for missing interpreters")
     parser.add_argument("--workdir", action="store",
@@ -408,6 +415,15 @@ def tox_addoption(parser):
         help="executable name or path of interpreter used to create a "
              "virtual test environment.")
 
+    def merge_description(testenv_config, value):
+        """the reader by default joins generated description with new line,
+         replace new line with space"""
+        return value.replace('\n', ' ')
+
+    parser.add_testenv_attribute(
+        name="description", type="string", default='', postprocess=merge_description,
+        help="short description of this environment")
+
     parser.add_testenv_attribute(
         name="envtmpdir", type="path", default="{envdir}/tmp",
         help="venv temporary directory")
@@ -416,15 +432,9 @@ def tox_addoption(parser):
         name="envlogdir", type="path", default="{envdir}/log",
         help="venv log directory")
 
-    def downloadcache(testenv_config, value):
-        if value:
-            # env var, if present, takes precedence
-            downloadcache = os.environ.get("PIP_DOWNLOAD_CACHE", value)
-            return py.path.local(downloadcache)
-
     parser.add_testenv_attribute(
-        name="downloadcache", type="string", default=None, postprocess=downloadcache,
-        help="(deprecated) set PIP_DOWNLOAD_CACHE.")
+        name="downloadcache", type="string", default=None,
+        help="(ignored) has no effect anymore, pip-8 uses local caching by default")
 
     parser.add_testenv_attribute(
         name="changedir", type="path", default="{toxinidir}",
@@ -458,12 +468,15 @@ def tox_addoption(parser):
             itertools.chain.from_iterable(
                 [x.split(' ') for x in value]))
 
-        passenv = set(["PATH", "PIP_INDEX_URL", "LANG", "LD_LIBRARY_PATH"])
+        passenv = set([
+            "PATH", "PIP_INDEX_URL", "LANG", "LANGUAGE", "LD_LIBRARY_PATH"
+        ])
 
         # read in global passenv settings
         p = os.environ.get("TOX_TESTENV_PASSENV", None)
         if p is not None:
-            passenv.update(x for x in p.split() if x)
+            env_values = [x for x in p.split() if x]
+            value.extend(env_values)
 
         # we ensure that tmp directory settings are passed on
         # we could also set it to the per-venv "envtmpdir"
@@ -473,6 +486,7 @@ def tox_addoption(parser):
             passenv.add("SYSTEMDRIVE")  # needed for pip6
             passenv.add("SYSTEMROOT")   # needed for python's crypto module
             passenv.add("PATHEXT")      # needed for discovering executables
+            passenv.add("COMSPEC")      # needed for distutils cygwincompiler
             passenv.add("TEMP")
             passenv.add("TMP")
         else:
@@ -504,11 +518,19 @@ def tox_addoption(parser):
     def sitepackages(testenv_config, value):
         return testenv_config.config.option.sitepackages or value
 
+    def alwayscopy(testenv_config, value):
+        return testenv_config.config.option.alwayscopy or value
+
     parser.add_testenv_attribute(
         name="sitepackages", type="bool", default=False, postprocess=sitepackages,
         help="Set to ``True`` if you want to create virtual environments that also "
              "have access to globally installed packages.")
 
+    parser.add_testenv_attribute(
+        name="alwayscopy", type="bool", default=False, postprocess=alwayscopy,
+        help="Set to ``True`` if you want virtualenv to always copy files rather "
+             "than symlinking.")
+
     def pip_pre(testenv_config, value):
         return testenv_config.config.option.pre or value
 
@@ -1045,6 +1067,10 @@ class Replacer:
             start, end = match.span()
             return match.string[start:end]
 
+        # special case: all empty values means ":" which is os.pathsep
+        if not any(g.values()):
+            return os.pathsep
+
         # special case: opts and packages. Leave {opts} and
         # {packages} intact, they are replaced manually in
         # _venv.VirtualEnv.run_install_command.
@@ -1146,7 +1172,8 @@ class _ArgvlistReader:
 
     @classmethod
     def processcommand(cls, reader, command):
-        posargs = getattr(reader, "posargs", None)
+        posargs = getattr(reader, "posargs", "")
+        posargs_string = list2cmdline([x for x in posargs if x])
 
         # Iterate through each word of the command substituting as
         # appropriate to construct the new command string. This
@@ -1155,12 +1182,11 @@ class _ArgvlistReader:
         newcommand = ""
         for word in CommandParser(command).words():
             if word == "{posargs}" or word == "[]":
-                if posargs:
-                    newcommand += " ".join(posargs)
+                newcommand += posargs_string
                 continue
             elif word.startswith("{posargs:") and word.endswith("}"):
                 if posargs:
-                    newcommand += " ".join(posargs)
+                    newcommand += posargs_string
                     continue
                 else:
                     word = word[9:-1]
@@ -1176,8 +1202,7 @@ class _ArgvlistReader:
         shlexer = shlex.shlex(newcommand, posix=True)
         shlexer.whitespace_split = True
         shlexer.escape = ''
-        argv = list(shlexer)
-        return argv
+        return list(shlexer)
 
 
 class CommandParser(object):
