@@ -237,7 +237,8 @@ class Reporter(object):
         self._reportedlines = []
         # self.cumulated_time = 0.0
 
-    def _get_verbosity(self):
+    @property
+    def verbosity(self):
         if self.session:
             return self.session.config.option.verbosity
         else:
@@ -268,11 +269,11 @@ class Reporter(object):
         self.tw.sep("_", "summary")
 
     def info(self, msg):
-        if self._get_verbosity() >= 2:
+        if self.verbosity >= 2:
             self.logline(msg)
 
     def using(self, msg):
-        if self._get_verbosity() >= 1:
+        if self.verbosity >= 1:
             self.logline("using %s" % (msg,), bold=True)
 
     def keyboard_interrupt(self):
@@ -308,15 +309,15 @@ class Reporter(object):
         self.tw.line("%s" % msg, **opts)
 
     def verbosity0(self, msg, **opts):
-        if self._get_verbosity() >= 0:
+        if self.verbosity >= 0:
             self.logline("%s" % msg, **opts)
 
     def verbosity1(self, msg, **opts):
-        if self._get_verbosity() >= 1:
+        if self.verbosity >= 1:
             self.logline("%s" % msg, **opts)
 
     def verbosity2(self, msg, **opts):
-        if self._get_verbosity() >= 2:
+        if self.verbosity >= 2:
             self.logline("%s" % msg, **opts)
 
     # def log(self, msg):
@@ -384,7 +385,9 @@ class Session:
         if self.config.option.showconfig:
             self.showconfig()
         elif self.config.option.listenvs:
-            self.showenvs()
+            self.showenvs(all_envs=False, description=self.config.option.verbosity > 0)
+        elif self.config.option.listenvs_all:
+            self.showenvs(all_envs=True, description=self.config.option.verbosity > 0)
         else:
             return self.subcommand_test()
 
@@ -625,9 +628,29 @@ class Session:
                 self.report.line("  %-15s = %s"
                                  % (attr.name, getattr(envconfig, attr.name)))
 
-    def showenvs(self):
-        for env in self.config.envlist:
-            self.report.line("%s" % env)
+    def showenvs(self, all_envs=False, description=False):
+        env_conf = self.config.envconfigs  # this contains all environments
+        default = self.config.envlist  # this only the defaults
+        extra = sorted([e for e in env_conf if e not in default]) if all_envs else []
+        if description:
+            self.report.line('default environments:')
+            max_length = max(len(env) for env in (default + extra))
+
+        def report_env(e):
+            if description:
+                text = env_conf[e].description or '[no description]'
+                msg = '{0} -> {1}'.format(e.ljust(max_length), text).strip()
+            else:
+                msg = e
+            self.report.line(msg)
+        for e in default:
+            report_env(e)
+        if all_envs:
+            if description:
+                self.report.line('')
+                self.report.line('additional environments:')
+            for e in extra:
+                report_env(e)
 
     def info_versions(self):
         versions = ['tox-%s' % tox.__version__]
